@@ -536,12 +536,15 @@ namespace SoftWire
 		// Debugging tools
 		void dumpSSE();
 
+		// Active code generator, used for variables stack
+		static CodeGenerator *activeCG();
+
 	private:
 		// Overloaded to detect modified/unmodified registers
 		virtual Encoding *x86(int instructionID,
-		                      const Operand &firstOperand = VOID,
-		                      const Operand &secondOperand = VOID,
-		                      const Operand &thirdOperand = VOID);
+		                      const Operand &firstOperand = Operand::OPERAND_VOID,
+		                      const Operand &secondOperand = Operand::OPERAND_VOID,
+		                      const Operand &thirdOperand = Operand::OPERAND_VOID);
 
 		void markModified(const Operand &op);   // Used to prevent spilling of unmodified registers
 		void retainLoad(const Operand &op);   // Retain the instruction that loads this register
@@ -562,9 +565,6 @@ namespace SoftWire
 		static const OperandMMREG &R64(Encoding::Reg r64);
 		static const OperandXMMREG &R128(Encoding::Reg r128);
 
-		// Active code generator, used for variables stack
-		static CodeGenerator *activeCG();
-
 		static CodeGenerator *active;
 
 		// Current allocation data
