@@ -8,8 +8,6 @@
 	inline int getch() {return fgetc(stdin);}
 #endif
 
-using namespace SoftWire;
-
 void testHelloWorld()
 {
 	printf("HelloWorld is a function which uses printf to print a message, ");
@@ -18,8 +16,8 @@ void testHelloWorld()
 	getch();
 	printf("Assembling HelloWorld.asm...\n\n");
 	
-	ASM_EXPORT(printf);
-	Assembler x86("HelloWorld.asm");
+	SoftWire::ASM_EXPORT(printf);
+	SoftWire::Assembler x86("HelloWorld.asm");
 
 	void (*helloWorld)() = (void(*)())x86.callable();
 
@@ -54,14 +52,14 @@ void testHelloUniverse()
 	printf("assembled from a source string.\n\n");
 	printf("Press any key to start assembling\n\n");
 	getch();
-	printf("Assembling HelloWorld.asm...\n\n");
+	printf("Assembling source string...\n\n");
 
 	const char* source = "string: DB \"Hello universe!\"\nHelloUniverse:\npush string\n"
 	                     "call printf\nadd esp, 4\nret";
 
-	ASM_EXPORT(printf);
+	SoftWire::ASM_EXPORT(printf);
 
-	Assembler x86(source, "HelloUniverse");
+	SoftWire::Assembler x86(source, "HelloUniverse");
 
 	void (*helloUniverse)() = (void(*)())x86.callable();
 
@@ -98,7 +96,7 @@ void testSetBits()
 	getch();
 	printf("Assembling SetBits.asm...\n\n");
 
-	Assembler x86("SetBits.asm");
+	SoftWire::Assembler x86("SetBits.asm");
 
 	void (*setBits)(unsigned int*, int, int) = (void(*)(unsigned int*, int, int))x86.callable();
 
@@ -136,7 +134,7 @@ void testCrossProduct()
 	getch();
 	printf("Assembling CrossProduct.asm...\n\n");
 
-	Assembler x86("CrossProduct.asm");
+	SoftWire::Assembler x86("CrossProduct.asm");
 
 	void (*crossProduct)(float*, float*, float*) = (void(*)(float*, float*, float*))x86.callable();
 
@@ -196,8 +194,8 @@ void testAlphaBlend()
 		katmai = false;
 	}
 
-	ASM_DEFINE(katmai);
-	Assembler x86("AlphaBlend.asm");
+	SoftWire::ASM_DEFINE(katmai);
+	SoftWire::Assembler x86("AlphaBlend.asm");
 
 	int (*alphaBlend)(int, int, int) = (int(*)(int, int, int))x86.callable();
 
@@ -233,7 +231,7 @@ void testFactorial()
 	getch();
 	printf("Assembling Factorial.asm...\n\n");
 
-	Assembler x86("Factorial.asm");
+	SoftWire::Assembler x86("Factorial.asm");
 
 	int (*factorial)(int) = (int(*)(int))x86.callable();
 
@@ -269,9 +267,9 @@ void testMandelbrot()
 	getch();
 	printf("Assembling Mandelbrot.asm...\n\n");
 
-	ASM_EXPORT(printf);
+	SoftWire::ASM_EXPORT(printf);
 
-	Assembler x86("Mandelbrot.asm");
+	SoftWire::Assembler x86("Mandelbrot.asm");
 
 	int (*mandelbrot)() = (int(*)())x86.callable();
 
@@ -306,13 +304,13 @@ void testIntrinsics()
 	printf("Press any key to start assembling\n\n");
 	getch();
 
-	Assembler x86;
+	SoftWire::Assembler x86;
 
 	static char string[] = "All working!";
 
 	x86.push((int)string);
 	x86.call((int)printf);
-	x86.add(esp, 4);
+	x86.add(x86.esp, 4);
 	x86.ret();
 
 	void (*emulator)() = (void(*)())x86.callable();
@@ -342,7 +340,7 @@ void testIntrinsics()
 	}
 }
 
-class TestRegisterAllocator : public CodeGenerator
+class TestRegisterAllocator : public SoftWire::CodeGenerator
 {
 public:
 	TestRegisterAllocator()
@@ -357,37 +355,37 @@ public:
 		x8 = 8;
 		x9 = 9;
 
-		static int t1 = 0;
-		static int t2 = 0;
-		static int t3 = 0;
-		static int t4 = 0;
-		static int t5 = 0;
-		static int t6 = 0;
-		static int t7 = 0;
-		static int t8 = 0;
-		static int t9 = 0;
+		Int t1;
+		Int t2;
+		Int t3;
+		Int t4;
+		Int t5;
+		Int t6;
+		Int t7;
+		Int t8;
+		Int t9;
 
 		pushad();
 
-		mov(x32(&t1), m32(&x1));
-		mov(x32(&t2), m32(&x2));
-		mov(x32(&t3), m32(&x3));
-		mov(x32(&t4), m32(&x4));
-		mov(x32(&t5), m32(&x5));
-		mov(x32(&t6), m32(&x6));
-		mov(x32(&t7), m32(&x7));
-		mov(x32(&t8), m32(&x8));
-		mov(x32(&t9), m32(&x9));
+		mov(t1, r32(&x1));
+		mov(t2, r32(&x2));
+		mov(t3, r32(&x3));
+		mov(t4, r32(&x4));
+		mov(t5, r32(&x5));
+		mov(t6, r32(&x6));
+		mov(t7, r32(&x7));
+		mov(t8, r32(&x8));
+		mov(t9, r32(&x9));
 
-		mov(dword_ptr [&x1], r32(&t9));
-		mov(dword_ptr [&x2], r32(&t8));
-		mov(dword_ptr [&x3], r32(&t7));
-		mov(dword_ptr [&x4], r32(&t6));
-		mov(dword_ptr [&x5], r32(&t5));
-		mov(dword_ptr [&x6], r32(&t4));
-		mov(dword_ptr [&x7], r32(&t3));
-		mov(dword_ptr [&x8], r32(&t2));
-		mov(dword_ptr [&x9], r32(&t1));
+		mov(dword_ptr [&x1], t9);
+		mov(dword_ptr [&x2], t8);
+		mov(dword_ptr [&x3], t7);
+		mov(dword_ptr [&x4], t6);
+		mov(dword_ptr [&x5], t5);
+		mov(dword_ptr [&x6], t4);
+		mov(dword_ptr [&x7], t3);
+		mov(dword_ptr [&x8], t2);
+		mov(dword_ptr [&x9], t1);
 
 		popad();
 		ret();
@@ -440,6 +438,71 @@ void testRegisterAllocator()
 	}
 }
 
+class TestOptimizations : public SoftWire::CodeGenerator
+{
+public:
+	TestOptimizations()
+	{
+		Int y;
+		Int z;
+
+		pushad();
+
+		mov(y, 1);
+		mov(z, y);
+		mov(dword_ptr [&x], z);
+
+		popad();
+		ret();
+	}
+
+	static x;
+};
+
+int TestOptimizations::x = 0;
+
+void testOptimizations()
+{
+	printf("Testing optimizations. SoftWire will eliminate redundant instructions.\n\n");
+	printf("Press any key to start assembling\n\n");
+	getch();
+
+	TestOptimizations x86;
+
+	void (*script)() = (void(*)())x86.callable();
+
+	if(script)
+	{
+		printf("%s\n\n", x86.getListing());
+		printf("Execute code (y/n)?\n\n");
+
+		int c;
+		do
+		{
+			c = getch();
+		}
+		while(c != 'y' && c != 'n');
+
+		if(c == 'y')
+		{
+			script();
+
+			if(x86.x == 1)
+			{
+				printf("Success!\n");				
+			}
+			else
+			{
+				printf("Failure! (%d)\n", x86.x);
+			}
+		}
+	}
+	else
+	{
+		printf(x86.getErrors());
+	}
+}
+
 int main()
 {
 	testHelloWorld();
@@ -451,6 +514,7 @@ int main()
 	testMandelbrot();
 	testIntrinsics();
 	testRegisterAllocator();
+	testOptimizations();
 
 	printf("Press any key to continue\n");
 	getch();
